/*
 *
 *  BlueZ D-Bus library
 *
 *  Copyright (C) 2005       Claudio Takahasi <claudio.takahasi@indt.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation;
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
 *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS,
 *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS
 *  SOFTWARE IS DISCLAIMED.
 *
 *
 *  $Id: client.c,v 0.1 2005/06/15 13:00:00 takahasi Exp $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <sys/syslog.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/stat.h>



#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/l2cap.h>
#include <bluetooth/bnep.h>
#include <bluetooth/dbus.h>

#include <dbus/dbus.h>
#include <dbus/dbus-protocol.h>

static DBusConnection *conn;

int init(void) 
{
	int ret = -1;
	DBusError error = {0};

	dbus_error_init(&error);

	conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);

        if (dbus_error_is_set (&error)) {
                syslog(LOG_ERR,"Get BUS error:(%s,%s)\n", error.name, error.message);
                dbus_error_free(&error);
	} else {
		ret = 0;
	}
		


	return ret;
}

static void cmd_name(const char *dev, int argc, char **argv) 
{
	DBusMessage *message = NULL;
	DBusMessage *reply = NULL;
	DBusMessageIter iter;
	DBusError error = {0};
	char path[64];
	const char *name;
	const char *bdaddr;

	bdaddr = argv[1];


	sprintf(path, "%s/%s/%s", MANAGER_PATH, dev, BLUEZ_HCI);

	message = dbus_message_new_method_call(BLUEZ_BUS_NAME, path, 
						MANAGER_INTERFACE, HCI_REMOTE_NAME);

	printf("%s,%s,%s, %s\n", path, MANAGER_INTERFACE, HCI_REMOTE_NAME, bdaddr);

	dbus_message_iter_init_append(message, &iter);
	dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING ,&bdaddr);

	dbus_error_init (&error);

	reply = dbus_connection_send_with_reply_and_block(conn, message, -1, &error);

        if (dbus_error_is_set (&error)){
		printf("[DBUS] reply error:(%s,%s)\n", error.name, error.message);
		dbus_error_free(&error); 
		goto failed; 
	}

	dbus_message_iter_init(reply, &iter);
	dbus_message_iter_get_basic(&iter, &name);

	printf("name: %s\n", name);

failed:
	if (message) dbus_message_unref(message);
	if (reply) dbus_message_unref(reply);

}

static void cmd_con(const char *dev, int argc, char **argv) 
{
	DBusMessage *message = NULL;
	DBusMessage *reply = NULL;
	DBusMessageIter iter;
	DBusMessageIter array_iter_read;
	DBusMessageIter struct_iter_read;
	DBusError error = {0};
	char path[64];
	uint16_t handle;
	const char *pbdaddr;
	uint8_t type;
	uint8_t out;
	uint16_t state;
	uint32_t link_mode;


	sprintf(path, "%s/%s/%s", MANAGER_PATH, dev, BLUEZ_HCI);

	message = dbus_message_new_method_call(BLUEZ_BUS_NAME, path, 
						MANAGER_INTERFACE, HCI_CONNECTIONS);

	printf("%s,%s,%s\n", path, MANAGER_INTERFACE, HCI_CONNECTIONS);

	dbus_error_init (&error);

	reply = dbus_connection_send_with_reply_and_block(conn, message, -1, &error);

        if (dbus_error_is_set (&error)){
		printf("[DBUS] reply error:(%s,%s)\n", error.name, error.message);
		dbus_error_free(&error); 
		goto failed; 
	}

	dbus_message_iter_init(reply, &iter);
	dbus_message_iter_recurse(&iter, &array_iter_read);

	do {
		dbus_message_iter_recurse(&array_iter_read, &struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &handle);
		dbus_message_iter_next(&struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &pbdaddr);
		dbus_message_iter_next(&struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &type);
		dbus_message_iter_next(&struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &out);
		dbus_message_iter_next(&struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &state);
		dbus_message_iter_next(&struct_iter_read);
		dbus_message_iter_get_basic(&struct_iter_read, &link_mode);

		printf("\t%s %s %s handle %d state %d lm %s\n", out ? "<" : ">",type == ACL_LINK ? "ACL" :"SCO",pbdaddr, handle, state, hci_lmtostr(link_mode));

	} while (dbus_message_iter_next(&array_iter_read));
	
	//dbus_message_iter_get_basic(&iter, &name);


failed:
	if (message) dbus_message_unref(message);
	if (reply) dbus_message_unref(reply);

}

static void print_help ()
{
	printf ("Client used to send D-BUS messages to the hcid\n" \
			"\thci_client <option> <command>\n" \
			"Options:\n" \
			"\t--help  -h                          Display all available commands\n" \
			"\t-i dev                              HCI device. eg: hci0, hci1\n" \
			"Commands:\n"\
			"\tname  <bdaddr>                      Show the remote device name\n" \
			"\tcon                                 Show active connections\n"\
	);
}

static struct {
	char *cmd;
	void (*func)(const char* dev, int argc, char **argv);
	char *doc;
} command[] = {
	{ "name",    cmd_name,    "get the remote name"                },
	{ "con",     cmd_con,     "Display active connections"         },
	{ NULL, NULL, 0 }
};


static struct option main_options[] = {
        { "help",       0, 0, 'h' },
        { "device",     1, 0, 'i' },
        { 0, 0, 0, 0 }
};

int main (int argc, char **argv) {
	int c , i;
	int opt;
	int digit_optind = 0;
	int8_t resp;
	char *iface = NULL;
	char *name;
	char *bdaddr;
	char dev[]= "default";

	init();

	while ((opt=getopt_long(argc, argv, "+i:h", main_options, NULL)) != -1) {
		switch (opt) { 
			case 'i':
				printf ("Using device: %s\n", optarg);
				strcpy(dev, optarg);
				break;
			case 'h':
			default:
				print_help(); 
				exit(0);
		}
	}

	argc -= optind;
	argv += optind;
	optind = 0;

	if (argc < 1) {
		print_help();
		exit(0);
	}


	for (i = 0; command[i].cmd; i++) {
		if (strncmp(command[i].cmd, argv[0], 3))
			continue;
		command[i].func(dev, argc, argv);
		break;
	}
cleanup:
	return 0;
}

